/* 
 * Autor:	Adam Bartczak, 5TIZ
 * Temat:	Biblioteka STL, Opis algorytmw : count count_if
 *
 * count count_if nale do algorytmw niemodyfikujcych STL
 *
 * template <class InputIterator, class T>
 *	 ptrdiff_t count ( InputIterator first, InputIterator last, const T& value );
 *
 * ptrdiff_t count_return = count(<element od ktrego zaczynamy wyszukiwanie>, 
 *										<element na ktrym koczymy wyszukiwanie>, <warot szukana>);
 *
 * Algorytm count oblicza licz elementw w kontenerze z zakresu [first, last) ktre pasuj do warotoci Value,
 * nastpnie zwraca liczb pasujcych elementw. 
 *
 * template <class InputIterator, class Predicate>
 *	 ptrdiff_t count_if ( InputIterator first, InputIterator last, Predicate pred );
 *
 * ptrdiff_t count_if_return = count_if(<element od ktrego zaczynamy wyszukiwanie>,
 *											 <element na ktrym koczymy wyszukiwanie>, <spenienie predykatu>);
 *
 * Algorytm count_if oblicza liczb elementw w kontenerze z zakresu [first, last) speniajcych predykat pred przekazany
 * jako obiekt funkcyjny, nastpnie zwraca liczb pasujcych elementw.
 *
 * Przykad zastosowania algorytmw count oraz count_if oparty jest na wyszukiwaniu odpowiednich elementw wrd vectora
 * zawierajcego polskie imiona.
 */
 
 #include <iostream>
 #include <vector>
 #include <string>
 // biblioteka z algorytmami STL
 #include <algorithm>
 // biblioteka wykorzystywana do dzielenia napisw
 #include <boost/tokenizer.hpp>
 
 // funkcja wykrywajca polskie imiona kobiet
 inline bool  NameWoman(const std::string& str) {
	return (str[str.size()-1] == 'a' || str[str.size()-1] == 'A');
 }
 
 // funkcja wykrywajca polskie imiona mczyzn
 inline bool  NameMan(const std::string& str) {
	return (!NameWoman(str));
 }
 
 // funkcja porwnujca ilo liter w sowie
 inline bool  NameSize(const std::string& str) {
	return (str.size() > 5);
 }
 
 int main() {
	
	typedef std::vector<std::string> VectorString;
	
	typedef VectorString::iterator VectorStringIt;
	
	// wektor imion
	VectorString vect;
	
	// iteratory wskazujace na wektor imion vect
	VectorStringIt start, end, it;
	
	// zmienna zawierajca liczb elementw pasujca do danego wzroca
	ptrdiff_t result;
	
	// przykladowy napis ktory jest wypelniony polskimi imionami, odzielony znakami
	// ktre nie s literami alfabetu (separatorami). W imionach nie naley wstawia 
	// polskich liter (,...).
	const std::string STRING_NAMES = "Adam Michal Bartek,   Adam . Ania ,/ EwA";
	
	// przykladowy wzorzec imienia dla funkcji count
	const std::string VALUE_COUNT = "Adam";

	// dzielenie napisu na poszczeglne imiona, nastpnie przypisywanie kadego z nich do wektora imion vect
	boost::tokenizer<> tok(STRING_NAMES);
	for(boost::tokenizer<>::iterator beg=tok.begin(); beg!=tok.end();++beg){
        vect.push_back(*beg);
    }
    
	// przypisanie iteratorom start i end wartoci pocztkowych
    start = vect.begin();
    
    end = vect.end();
    
    // liczba wystapien imienia zawartego w zmiennej VALUE_COUNT (zastosowanie algorymu count)
    result = count(start, end, VALUE_COUNT) ;

    std::cout << "Liczba wystapien imienia " << VALUE_COUNT << ": " << result << std::endl;
    
    // liczba imion eskich w wektorze imion (zastosowanie algorymu count_if)
    result = count_if(start, end, NameWoman) ;
    
    std::cout << "Liczba Kobiet: " << result << std::endl;
    
    // liczba imion mskich w wektorze imion (zastosowanie algorymu count_if)
    result = count_if(start, end, NameMan) ;
    
    std::cout << "Liczba Mezczyzn: " << result << std::endl;
    
    // liczba imion zawierajce wicej ni 5 liter (zastosowanie algorymu count_if)
    result = count_if(start, end, NameSize) ;
    
    std::cout << "Liczba slow o liczbie liter > 5: " << result << std::endl;
    
    getchar();

	return 0;
 }
